<?php
/**
 * Created by PhpStorm.
 * User: wanderleijunior
 * Date: 12/07/19
 * Time: 10:18
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Teknisa\Model\Teknisa\Repositories\OmProduto;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Produto
{

    /** @var EntityManager **/
    protected $entityManager;

    public function __construct(EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function findCollectionVariableMeasures($conditions, $dataSourceName, $page, $pageSize, $orderBy, $nrProdutoId, $nrOrgId, $cdServOrca)
    {
        $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
        /** @var OmProduto $repositoryOmProduto */
        $repositoryOmProduto = InstanceProvider::getRepository()->getOmProduto();
        $conditions = $conditionsFixed['CONDITIONS'];
        $zmartFilterCriteria = InstanceProvider::getUtilities()->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
        if(empty($nrProdutoId)) {
            $productId = '0';
            $zmartFilterCriteria->addCondition('serviceId', '=', $cdServOrca);
        } else {
            /** @var \Teknisa\Model\Teknisa\Entities\OmProduto $produto */
            $produto   = $repositoryOmProduto->find($nrProdutoId);
            $productId = strval($produto->getNrexternalProductId());
        }
        $zmartFilterCriteria->addCondition('productId', '=', $productId);
        $zmartFilterCriteria->setPage($page);
        $zmartFilterCriteria->setPageSize($pageSize);

        if (empty($orderBy)) {
            $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
        } else {
            foreach ($orderBy as $idx => $order) {
                $zmartFilterCriteria->addOrderBy($idx, $order);
            }
        }
        $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
        $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
        $registers = InstanceProvider::getLicenseMongoDB()->aggregate("VariableMeasure_{$nrOrgId}", $criteria);
        foreach ($registers as &$register) {
            $register['date']           = \DateTime::createFromFormat('Ymd', $register['date'])          ->format('d/m/Y');
            $register['endCompetence']  = \DateTime::createFromFormat('Ymd', $register['endCompetence']) ->format('d/m/Y');
            $register['initCompetence'] = \DateTime::createFromFormat('Ymd', $register['initCompetence'])->format('d/m/Y');
            $contractId  = Utilities::getValueInFilterDataSourceByName($conditions, 'contractId');
            $register['countMin'] = $repositoryOmProduto->getMinContract($contractId, $nrProdutoId, $cdServOrca);

        }
        return $registers;
    }

    private function fixConditionsToZmartFilterCriteria($conditions) {
        $mappedLikeAll = null;
        foreach($conditions as $idx => &$condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            if($condition['operator'] == 'LIKE_I' || $condition['operator'] == 'LIKE') {
                $condition['operator'] = 'LIKE_ALL';
            }
            if($name == 'date') {
                $condition['value'] = array(
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][0])->format('Ymd')),
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][1])->format('Ymd'))
                );
            } else if($name == 'initCompetence' || $name == 'endCompetence'){
                $condition['value'] = intval(\DateTime::createFromFormat('d/m/Y H:i:s', $condition['value'])->format('Ymd'));
            }
            if ($condition['operator'] == 'MAPPED_LIKE_ALL') {
                $mappedLikeAll = $condition;
                unset($conditions[$idx]);
            }
        }
        return array('CONDITIONS' => $conditions, 'MAPPED_LIKE_ALL' => $mappedLikeAll);
    }

    private function setMappedLikeAllCriteria(&$criteria, $mappedLikeAll) {
        if(!empty($mappedLikeAll)) {
            if (!isset($criteria[0]['$match'])) {
                $criteria['$match'] = array();
            }
            $name = isset($mappedLikeAll['name']) ? $mappedLikeAll['name'] : $mappedLikeAll['columnName'];
            $mappedLikeAllColumns = array('measureName', 'contractId', 'count', 'operatorName', 'serviceName');
            $value = str_replace('%', '', $mappedLikeAll['value'][0]['_ALL']);
            foreach ($mappedLikeAllColumns as $column) {
                $criteria[0]['$match']['$or'][] = array($column => array('$regex' => ".*{$value}.*"));
            }
        }
    }

    /**
     * @param $conditions
     * @param $nrparcnegocio
     * @return array
     */
    public function findProdutosContratados($conditions, $nrparcnegocio)
    {
        $filter = InstanceProvider::getUtilities()->createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getOmProduto()->findProdutosContratados($filter, $nrparcnegocio);
    }

    public function findproductsTranslations($language)
    {
        $org          = Utilities::getEnvironment()->getCurrentCustomerOrganization();
        $translations = InstanceProvider::getRepository()->getOmProduto()->findproductsTranslations($org->getNrorg(), $language);
        return array_column($translations, 'TRANSLATE', 'WORD');
    }
}
